
#include <windows.h>  
#include <stdio.h>
#include "tables.h"
#include "sqlext.h"  


#if defined (WIN32)
	#define IS_WIN32 TRUE
#else
	#define IS_WIN32 FALSE
#endif

#define IS_NT      IS_WIN32 && (BOOL)(GetVersion() < 0x80000000)
#define IS_WIN32S  IS_WIN32 && (BOOL)(!(IS_NT) && (LOBYTE(LOWORD(GetVersion()))<4))
#define IS_WIN95   (BOOL)(!(IS_NT) && !(IS_WIN32S)) && IS_WIN32

HINSTANCE hInst;   // current instance

LPCTSTR lpszAppName = "MyApp";
LPCTSTR lpszTitle   = "SQLTables()"; 


BOOL RegisterWin95( CONST WNDCLASS* lpwc );


int APIENTRY WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
                      LPTSTR lpCmdLine, int nCmdShow)
{
   MSG      msg;
   HWND     hWnd; 
   WNDCLASS wc;

   wc.style         = CS_HREDRAW | CS_VREDRAW;
   wc.lpfnWndProc   = (WNDPROC)WndProc;       
   wc.cbClsExtra    = 0;                      
   wc.cbWndExtra    = 0;                      
   wc.hInstance     = hInstance;              
   wc.hIcon         = LoadIcon (hInstance, lpszAppName); 
   wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
   wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
   wc.lpszMenuName  = lpszAppName;              
   wc.lpszClassName = lpszAppName;              

   if ( IS_WIN95 )
   {
      if ( !RegisterWin95( &wc ) )
         return( FALSE );
   }
   else if ( !RegisterClass( &wc ) )
      return( FALSE );

   hInst = hInstance; 

   hWnd = CreateWindow( lpszAppName, 
                        lpszTitle,    
                        WS_OVERLAPPEDWINDOW, 
                        CW_USEDEFAULT, 0, 
                        CW_USEDEFAULT, 0,  
                        NULL,              
                        NULL,              
                        hInstance,         
                        NULL               
                      );

   if ( !hWnd ) 
      return( FALSE );

   ShowWindow( hWnd, nCmdShow ); 
   UpdateWindow( hWnd );         

   while( GetMessage( &msg, NULL, 0, 0) )   
   {
      TranslateMessage( &msg ); 
      DispatchMessage( &msg );  
   }

   return( msg.wParam ); 
}


BOOL RegisterWin95( CONST WNDCLASS* lpwc )
{
   WNDCLASSEX wcex;

   wcex.style         = lpwc->style;
   wcex.lpfnWndProc   = lpwc->lpfnWndProc;
   wcex.cbClsExtra    = lpwc->cbClsExtra;
   wcex.cbWndExtra    = lpwc->cbWndExtra;
   wcex.hInstance     = lpwc->hInstance;
   wcex.hIcon         = lpwc->hIcon;
   wcex.hCursor       = lpwc->hCursor;
   wcex.hbrBackground = lpwc->hbrBackground;
   wcex.lpszMenuName  = lpwc->lpszMenuName;
   wcex.lpszClassName = lpwc->lpszClassName;

   // Added elements for Windows 95.
   //...............................
   wcex.cbSize = sizeof(WNDCLASSEX);
   wcex.hIconSm = LoadImage(wcex.hInstance, lpwc->lpszClassName, 
                            IMAGE_ICON, 16, 16,
                            LR_DEFAULTCOLOR );
			
   return RegisterClassEx( &wcex );
}


LRESULT CALLBACK WndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
static HENV  hEnv  = NULL;
static HDBC  hDBC  = NULL;
static HWND  hList = NULL;

   switch( uMsg )
   {
      case WM_CREATE :
              {
                 RETCODE retCode;

                 // Allocate Environment and Connection.
                 //.....................................
                 SQLAllocEnv( &hEnv );
                 SQLAllocConnect( hEnv, &hDBC );

                 // Connect to the data source
                 //...........................
                 retCode = SQLConnect( hDBC, 
                             "Test Data Source", SQL_NTS,
                             "DBA",              SQL_NTS,  
                             "SQL",              SQL_NTS );

                 if ( retCode == SQL_SUCCESS )
                 {
                    int nTabSize = 60;

                    hList = CreateWindow( "LISTBOX", "",    
                               LBS_NOTIFY | LBS_USETABSTOPS |
                               LBS_NOINTEGRALHEIGHT  | LBS_SORT | 
                               WS_BORDER  | WS_CHILD | 
                               WS_VISIBLE | WS_VSCROLL, 
                               0, 0, 0, 0, 
                               hWnd, (HMENU)101,
                               hInst, NULL );
                    
                    SendMessage( hList, LB_SETTABSTOPS, 
                                 1, (LPARAM)&nTabSize );
                 }
              }
              break;

      case WM_SIZE :
              MoveWindow( hList, 0, 0, LOWORD( lParam ), 
                                       HIWORD( lParam ), TRUE );
              break; 
              
      case WM_COMMAND :
              switch( LOWORD( wParam ) )
              {
                 case IDM_TEST :
                        {
                           HSTMT   hStmt;
                           UCHAR   szTableOwner[128];
                           UCHAR   szTableName [128];
                           UCHAR   szLBStr     [128];
                           SDWORD  dwTypeLen;
                           RETCODE retCode;

                           // Allocate a statement handle
                           // to receive the result set.
                           // ...........................
                           SQLAllocStmt( hDBC, &hStmt );

                           // Call SQLTables() to determine all table  
                           // names accessible from the data source.
                           // ......................................
                           retCode = SQLTables( hStmt, 
                                        NULL,      SQL_NTS, 
                                        "DBA",     SQL_NTS, 
                                        NULL,      SQL_NTS, 
                                        "'TABLE'", SQL_NTS ); 

                           if( retCode == SQL_SUCCESS || 
                               retCode == SQL_SUCCESS_WITH_INFO )
                           {
                              // Bind a column to the TABLE_OWNER and 
                              // TABLE_NAME columns in the result set 
                              // generated from SQLTables().
                              // ....................................
                              SQLBindCol( hStmt, 2, SQL_C_CHAR, 
                                 szTableOwner, sizeof( szTableOwner ), 
                                 &dwTypeLen );

                              SQLBindCol( hStmt, 3, SQL_C_CHAR,
                                 szTableName, sizeof( szTableName ), 
                                 &dwTypeLen );

                              // Fetch all the records and
                              // display them in the list box.
                              //..............................
                              retCode = SQLFetch( hStmt );

                              while ( retCode == SQL_SUCCESS )
                              {
                                 // set string contents to be
                                 // the new owner and name
                                 // .........................
                                 szLBStr[0] = '\0';
                                 strcat( szLBStr, szTableOwner );
                                 strcat( szLBStr, "\t" );
                                 strcat( szLBStr, szTableName );

                                 SendMessage( hList, LB_ADDSTRING, 
                                              0, (LPARAM)szLBStr );

                                 retCode = SQLFetch( hStmt );
                              }
                           }

                           SQLFreeStmt( hStmt, SQL_DROP );
                        }
                        break;

                 case IDM_ABOUT :
                        DialogBox( hInst, "AboutBox", hWnd, (DLGPROC)About );
                        break;

                 case IDM_EXIT :
                        DestroyWindow( hWnd );
                        break;
              }
              break;
      
      case WM_DESTROY :
              PostQuitMessage(0);

              // Disconnect Environment.
              //........................
              SQLDisconnect( hDBC );

              // Free Connection and Environment.
              //.................................
              SQLFreeConnect( hDBC );
              SQLFreeEnv( hEnv );
              break;

      default :
            return( DefWindowProc( hWnd, uMsg, wParam, lParam ) );
   }

   return( 0L );
}


LRESULT CALLBACK About( HWND hDlg,           
                        UINT message,        
                        WPARAM wParam,       
                        LPARAM lParam)
{
   switch (message) 
   {
       case WM_INITDIALOG: 
               return (TRUE);

       case WM_COMMAND:                              
               if (   LOWORD(wParam) == IDOK         
                   || LOWORD(wParam) == IDCANCEL)    
               {
                       EndDialog(hDlg, TRUE);        
                       return (TRUE);
               }
               break;
   }

   return (FALSE); 
}
